Package: hypertext
Title: 'HTML' Element Construction
Version: 1.3.0
Authors@R: 
    c(
      person("Kennedy", "Mwavu", , "mwavukennedy@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0006-3157-7234")),
      person("Sigflux", role = c("cph", "fnd"))
    )
Description: Provides a deterministic, framework-agnostic Domain-Specific Language for building
    'HTML' nodes and rendering them to a string.
License: MIT + file LICENSE
URL: https://sigflux.github.io/hypertext/,
        https://github.com/sigflux/hypertext
BugReports: https://github.com/sigflux/hypertext/issues
Encoding: UTF-8
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
Config/Needs/website: pkgdown
NeedsCompilation: no
Packaged: 2026-06-19 18:55:55 UTC; mwavu
Author: Kennedy Mwavu [aut, cre] (ORCID:
    <https://orcid.org/0009-0006-3157-7234>),
  Sigflux [cph, fnd]
Maintainer: Kennedy Mwavu <mwavukennedy@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-21 14:40:13 UTC
Built: R 4.6.0; ; 2026-06-21 15:37:50 UTC; unix
