# Generated by roxygen2: do not edit by hand

S3method(print,hypertext.raw)
S3method(print,hypertext.tag)
S3method(print,hypertext.tag.list)
S3method(render,default)
S3method(render,hypertext.raw)
S3method(render,hypertext.tag)
S3method(render,list)
export(doctype)
export(raw_html)
export(render)
export(tag)
export(tag_list)
export(tags)
